@@ -22,19 +22,34 @@ from servo.types import Adjustment, Control, Description, Duration, Measurement
 from servo.servo import _set_current_servo
 
 
-class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
-    servo: servo.Servo
-    connected: bool = False
-    _running: bool = False
+class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
+    _servo: servo.Servo = pydantic.PrivateAttr(None)
+    _connected: bool = pydantic.PrivateAttr(False)
+    _running: bool = pydantic.PrivateAttr(False)
+    _main_loop_task: Optional[asyncio.Task] = pydantic.PrivateAttr(None)
 
-    def __init__(self, servo_: servo) -> None: # noqa: D107
-        self.servo = servo_
+    class Config:
+        arbitrary_types_allowed = True
+
+    def __init__(self, servo_: servo) -> None: # noqa: D10
+        super().__init__()
+        self._servo = servo_
 
         # initialize default servo options if not configured
         if self.config.servo is None:
             self.config.servo = servo.ServoConfiguration()
 
-        super().__init__()
+    @property
+    def servo(self) -> servo.Servo:
+        return self._servo
+
+    @property
+    def running(self) -> bool:
+        return self._running
+
+    @property
+    def connected(self) -> bool:
+        return self._connected
 
     @property
     def optimizer(self) -> servo.Optimizer:
@@ -95,7 +110,7 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
 
     @backoff.on_exception(
         backoff.expo,
-        httpx.HTTPError,
+        (httpx.HTTPError, pydantic.ValidationError),
         max_time=lambda: servo.current_servo().config.servo.backoff.max_time(),
         max_tries=lambda: servo.current_servo().config.servo.backoff.max_tries(),
     )
@@ -106,7 +121,7 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
 
         if cmd_response.command == servo.api.Commands.describe:
             description = await self.describe()
-            self.logger.info(
+            self.logger.success(
                 f"Described: {len(description.components)} components, {len(description.metrics)} metrics"
             )
             self.logger.trace(devtools.pformat(description))
@@ -116,7 +131,7 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
 
         elif cmd_response.command == servo.api.Commands.measure:
             measurement = await self.measure(cmd_response.param)
-            self.logger.info(
+            self.logger.success(
                 f"Measured: {len(measurement.readings)} readings, {len(measurement.annotations)} annotations"
             )
             self.logger.trace(devtools.pformat(measurement))
@@ -135,14 +150,13 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
                 settings_count = sum(
                     len(component.settings) for component in description.components
                 )
-                self.logger.info(
+                self.logger.success(
                     f"Adjusted: {components_count} components, {settings_count} settings"
                 )
             except servo.AdjustmentFailedError as error:
+                self.logger.info(f"Adjustment failed: {error}")
                 status = servo.api.Status.from_error(error)
-                self.logger.error(
-                    f"Adjustment failed: {error}"
-                )
+                self.logger.opt(exception=error).debug("Adjust failure details")
 
             return await self._post_event(servo.api.Events.adjust, status.dict())
 
@@ -174,13 +188,35 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
                     self.logger.warning(
                         f"server reported unexpected event: {status.reason}"
                     )
+
             except (httpx.TimeoutException, httpx.HTTPStatusError) as error:
-                self.logger.warning(f"ignoring HTTP timeout error: {error}")
+                self.logger.warning(f"command execution failed HTTP client error: {error}")
+
+            except pydantic.ValidationError as error:
+                self.logger.warning(f"command execution failed with model validation error: {error}")
+                self.logger.opt(exception=error).debug("Pydantic model failed validation")
 
             except Exception as error:
                 self.logger.exception(f"failed with unrecoverable error: {error}")
                 raise error
 
+    def run_main_loop(self) -> None:
+        if self._main_loop_task:
+            self._main_loop_task.cancel()
+
+        def _reraise_if_necessary(task: asyncio.Task) -> None:
+            try:
+                task.result()
+            except asyncio.CancelledError:
+                raise  # Task cancellation should not be logged as an error.
+            except Exception as error:  # pylint: disable=broad-except
+                self.logger.error(f"Exiting from servo main loop do to error: {error} (task={task})")
+                self.logger.opt(exception=error).trace(f"Exception raised by task {task}")
+                raise error  # Ensure that we surface the error for handling
+
+        self._main_loop_task = asyncio.create_task(self.main_loop(), name=f"main loop for servo {self.optimizer.id}")
+        self._main_loop_task.add_done_callback(_reraise_if_necessary)
+
     async def run(self) -> None:
         self._running = True
 
@@ -206,17 +242,18 @@ class ServoRunner(servo.logging.Mixin, servo.api.Mixin):
             async def connect() -> None:
                 self.logger.info("Saying HELLO.", end=" ")
                 await self._post_event(servo.api.Events.hello, dict(agent=servo.api.user_agent()))
-                self.connected = True
+                self._connected = True
 
 
             self.logger.info(f"Connecting to Opsani Optimizer @ {self.optimizer.api_url}...")
             await connect()
-        except asyncio.CancelledError:
-            pass
+        except asyncio.CancelledError as error:
+            self.logger.opt(exception=error).trace("task cancelled, aborting servo runner")
+            raise error
         except:
-            servo.logger.exception("exception encountered during connect")
+            self.logger.exception("exception encountered during connect")
 
-        asyncio.create_task(self.main_loop(), name="main loop")
+        self.run_main_loop()
 
     async def shutdown(self, *, reason: Optional[str] = None) -> None:
         """Shutdown the running servo."""
@@ -231,6 +268,8 @@ class AssemblyRunner(pydantic.BaseModel, servo.logging.Mixin):
     assembly: servo.Assembly
     runners: List[ServoRunner] = []
     progress_handler: Optional[servo.logging.ProgressHandler] = None
+    progress_handler_id: Optional[int] = None
+    _running: bool = pydantic.PrivateAttr(False)
 
     class Config:
         arbitrary_types_allowed = True
@@ -245,11 +284,19 @@ class AssemblyRunner(pydantic.BaseModel, servo.logging.Mixin):
 
         raise KeyError(f"no runner was found for the servo: \"{servo}\"")
 
+    @property
+    def running(self) -> bool:
+        return self._running
+
     def run(self) -> None:
         """Asynchronously run all servos active within the assembly.
 
         Running the assembly takes over the current event loop and schedules a `ServoRunner` instance for each servo active in the assembly.
         """
+        if self.running:
+            raise RuntimeError("Cannot run an assembly that is already running")
+
+        self._running = True
         loop = asyncio.get_event_loop()
 
         # Setup signal handling
@@ -292,12 +339,16 @@ class AssemblyRunner(pydantic.BaseModel, servo.logging.Mixin):
 
                 # Restart a fresh main loop
                 runner = self._runner_for_servo(servo.current_servo())
-                asyncio.create_task(runner.main_loop(), name="main loop")
+                runner.run_main_loop()
+            else:
+                self.logger.error(
+                    f"unrecognized exception passed to progress exception handler: {error}"
+                )
 
         self.progress_handler = servo.logging.ProgressHandler(
             _report_progress, self.logger.warning, handle_progress_exception
         )
-        self.logger.add(self.progress_handler.sink, catch=True)
+        self.progress_handler_id = self.logger.add(self.progress_handler.sink, catch=True)
 
         self._display_banner()
 
@@ -375,34 +426,68 @@ class AssemblyRunner(pydantic.BaseModel, servo.logging.Mixin):
         secho(reset=True)
 
     async def _shutdown(self, loop, signal=None):
+        if not self.running:
+            raise RuntimeError("Cannot shutdown an assembly that is not running")
+
         if signal:
             self.logger.info(f"Received exit signal {signal.name}...")
 
         reason = signal.name if signal else "shutdown"
-        self.logger.info(f"Shutting down {len(self.runners)} running servos...")
-        for runner in self.runners:
-            await runner.shutdown(reason=reason)
 
-        self.logger.info("Dispatching shutdown event...")
-        await self.assembly.shutdown()
-        await self.progress_handler.shutdown()
+        # Shut down the servo runners, breaking active control loops
+        if len(self.runners) == 1:
+            self.logger.info(f"Shutting down servo...")
+        else:
+            self.logger.info(f"Shutting down {len(self.runners)} running servos...")
+        for fut in asyncio.as_completed(list(map(lambda r: r.shutdown(reason=reason), self.runners)), timeout=30.0):
+            try:
+                await fut
+            except Exception as error:
+                self.logger.critical(f"Failed servo runner shutdown with error: {error}")
 
+        # Shutdown the assembly and the servos it contains
+        self.logger.debug("Dispatching shutdown event...")
+        try:
+            await self.assembly.shutdown()
+        except Exception as error:
+            self.logger.critical(f"Failed assembly shutdown with error: {error}")
+
+        await asyncio.gather(self.progress_handler.shutdown(), return_exceptions=True)
+        self.logger.remove(self.progress_handler_id)
+
+        # Cancel any outstanding tasks -- under a clean, graceful shutdown this list will be empty
+        # The shutdown of the assembly and the servo should clean up its tasks
         tasks = [t for t in asyncio.all_tasks() if t is not asyncio.current_task()]
+        if len(tasks):
+            [task.cancel() for task in tasks]
 
-        [task.cancel() for task in tasks]
-
-        self.logger.info(f"Cancelling {len(tasks)} outstanding tasks")
-        await asyncio.gather(*tasks, return_exceptions=True)
+            self.logger.info(f"Cancelling {len(tasks)} outstanding tasks")
+            self.logger.debug(f"Outstanding tasks: {devtools.pformat(tasks)}")
+            await asyncio.gather(*tasks, return_exceptions=True)
 
         self.logger.info("Servo shutdown complete.")
-        await self.logger.complete()
+        await asyncio.gather(self.logger.complete(), return_exceptions=True)
+
+        self._running = False
 
         loop.stop()
 
     def _handle_exception(self, loop: asyncio.AbstractEventLoop, context: dict) -> None:
-        self.logger.error(f"asyncio exception handler triggered with context: {context}")
+        self.logger.critical(f"asyncio exception handler triggered with context: {context}")
 
-        self.logger.critical(
-            "Shutting down due to unhandled exception in asyncio event loop..."
-        )
-        asyncio.create_task(self._shutdown(loop))
+        exception = context.get("exception", None)
+        logger = self.logger.opt(exception=exception)
+
+        if loop.is_closed():
+            logger.critical(
+                "Ignoring exception -- the event loop is closed."
+            )
+        elif self.running:
+            logger.critical(
+                "Shutting down due to unhandled exception in asyncio event loop..."
+            )
+            loop.create_task(self._shutdown(loop))
+        else:
+            logger.critical(
+                "Ignoring exception -- the assembly is not running"
+            )
